#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "ppport.h"

#include <cmae_client.h>

/*
 * Propagate an envelope attribute from an HV to a CMAE_Envelope object.
 * Uses and sets RETVAL in addition to its parameter.
 */

#define GET_ENVELOPE_ATTR(HV, ATTR, VAR, GETTER)                    \
{                                                                   \
    SV **e_svp = hv_fetch((HV), #ATTR, sizeof(#ATTR)-1, 0);         \
    if (e_svp)                                                      \
       (VAR) = GETTER(*e_svp);                                      \
}                                                                   \


/*
 * If the string N is equal to the name of X, then set RETVAL to the
 * value of X.
 *
 * NOTE: This must take the form of an 'if' so can chain it with 'else'.
 */

#define TRY_CONST(N,X)  if (!(strcmp((N), #X))) { RETVAL = X; }


/*----------------------------------------------------------------
 * XS code begins
 *
 * NOTE NOTE NOTE that the second PREFIX is "CMAE_Client", not "CMAE_Client_".
 * This has the salutory effect of marking all these methods with a leading
 * underscore in Perl, which signifies that they're to be considered private.
 */

MODULE = Cloudmark::CMAE::Client        PACKAGE = Cloudmark::CMAE::Client


const char *
CMAE_StrError(CMAE_ERR err)


MODULE = Cloudmark::CMAE::Client        PACKAGE = Cloudmark::CMAE::Client       PREFIX = CMAE_Client


const char *
CMAE_Client_Library_Version()
    CODE:
        RETVAL = CMAE_Client_Version;
    OUTPUT:
        RETVAL


unsigned short
CMAE_Client_Default_Port()


CMAE_ERR
CMAE_Client_Open(SV *SessionOut,                                \
                 const char *ServerName,                        \
                 unsigned long ServerIPAddr,                    \
                 unsigned short ServerPort = 0,                 \
                 unsigned ConnectTimeout = 0)
    PROTOTYPE: DISABLE
    PREINIT:
        CMAE_Client_Session sess;
    CODE:
        sess = NULL;
        RETVAL = CMAE_Client_Open(&sess, ServerName, ServerIPAddr, ServerPort, ConnectTimeout);
        if (!RETVAL)
            sv_setref_pv(SessionOut, "Cloudmark::CMAE::Client", (void*)sess);
    OUTPUT:
        SessionOut
        RETVAL


CMAE_ERR
CMAE_Client_Close(CMAE_Client_Session Session)


CMAE_ERR
CMAE_Client_SetProtocolTimeout(CMAE_Client_Session Session, \
                               unsigned ProtocolTimeout)


CMAE_ERR
CMAE_Client_GetProtocolTimeout(CMAE_Client_Session Session, \
                               OUT unsigned ProtocolTimeoutOut)


CMAE_ERR
CMAE_Client_GetMaxMessageSize(CMAE_Client_Session Session, \
                              OUT size_t MaxMessageSizeOut)


CMAE_ERR
CMAE_Client_Score(CMAE_Client_Session Session,    \
                  HV *Envelope,                   \
                  SV *RFC822Content,              \
                  OUT unsigned ScoreOut,          \
                  OUT unsigned CategoryOut,       \
                  OUT unsigned SubCategoryOut,    \
                  OUT unsigned RescanOut,         \
                  SV *AnalysisOut)
    PREINIT:
        STRLEN rfc822_len;
        char *rfc822_ptr;
        CMAE_Envelope env = NULL;
        char *analysis;

    CODE:
        RETVAL = 0;

        rfc822_ptr = SvPV(RFC822Content, rfc822_len);

        /* If the provided Envelope hash has any elements, we'll pass a
           non-NULL envelope to CMAE_Client_Score(). */
        if (hv_iterinit(Envelope)) {
            char *e_FromIPStr = NULL;
            struct in_addr e_FromIP;
            char *e_FromHost = NULL;
            char *e_Helo = NULL;
            char *e_MailFrom = NULL;
            const char **e_RcptTos = NULL;
            unsigned int e_NumRcptTos = 0;
            SV **svp;

            GET_ENVELOPE_ATTR(Envelope, FromIP,   e_FromIPStr, SvPV_nolen);
            GET_ENVELOPE_ATTR(Envelope, FromHost, e_FromHost,  SvPV_nolen);
            GET_ENVELOPE_ATTR(Envelope, Helo,     e_Helo,      SvPV_nolen);
            GET_ENVELOPE_ATTR(Envelope, MailFrom, e_MailFrom,  SvPV_nolen);

            memset(&e_FromIP, 0, sizeof e_FromIP);
            if (e_FromIPStr)
                inet_aton(e_FromIPStr, &e_FromIP);

            /* <perl_barbie> Processing arrays is hard. </perl_barbie> */

            svp = hv_fetch(Envelope, "RcptTo", 6, 0);
            if (svp) {
                SV *rv = *svp;
                AV *av;
                I32 len, i;

                if (!SvROK(rv) || SvTYPE(SvRV(rv)) != SVt_PVAV)
                   die("CMAE envelope: 'RcptTo' must be an array");

                av = (AV*)SvRV(rv);
                len = av_len(av);
                e_NumRcptTos = len + 1;
                e_RcptTos = (const char **) malloc(e_NumRcptTos * sizeof(const char *));
                for (i = 0; i <= len; ++i) {
                    svp = av_fetch(av, i, 0);
                    if (!svp)
                        die("CMAE envelope: 'RcptTo' array fetch failed");
                    e_RcptTos[i] = SvPV_nolen(*svp);
                }
            }

            RETVAL = CMAE_EnvelopeInit(&env, e_FromIP.s_addr, e_FromHost, e_Helo,
                                       e_MailFrom, e_RcptTos, e_NumRcptTos);
            if (RETVAL)
                env = NULL;

            if (e_RcptTos)
                free(e_RcptTos);
        }

        if (!RETVAL) {
            RETVAL = CMAE_Client_Score(Session, env, rfc822_ptr, rfc822_len,
                                       &ScoreOut, &CategoryOut, &SubCategoryOut, &RescanOut, &analysis);
            if (!RETVAL) {
                sv_setpv(AnalysisOut, analysis);
                CMAE_Client_Free(analysis);
            }
        }

        if (env)
            CMAE_EnvelopeDone(env);

    OUTPUT:
        ScoreOut
        CategoryOut
        SubCategoryOut
        RescanOut
        AnalysisOut
        RETVAL


CMAE_ERR
CMAE_Client_Score_Analysis(CMAE_Client_Session Session,    \
                           SV *Analysis,                   \
                           OUT unsigned ScoreOut,          \
                           OUT unsigned CategoryOut,       \
                           OUT unsigned SubCategoryOut,    \
                           OUT unsigned RescanOut)
    CODE:
        RETVAL = CMAE_Client_Score_Analysis(Session, SvPV_nolen(Analysis),
                                            &ScoreOut, &CategoryOut, &SubCategoryOut, &RescanOut);

    OUTPUT:
        ScoreOut
        CategoryOut
        SubCategoryOut
        RescanOut
        RETVAL


CMAE_ERR
CMAE_Client_DescribeCategory(CMAE_Client_Session Session,  \
                             unsigned CategoryOut,         \
                             unsigned SubCategoryOut,      \
                             OUT char *CategoryDescOut,    \
                             OUT char *SubCategoryDescOut)


CMAE_ERR
CMAE_Client_GetCartridgeVersion(CMAE_Client_Session Session, \
                                SV *VersionOut)
    PREINIT:
        char *version;
    CODE:
        RETVAL = CMAE_Client_GetCartridgeVersion(Session, &version);
        if (!RETVAL) {
            sv_setpv(VersionOut, version);
            CMAE_Client_Free(version);
        }
    OUTPUT:
        RETVAL

int
_constant(const char *name)
    CODE:
             TRY_CONST(name, CMAE_ERR_NONE)
        else TRY_CONST(name, CMAE_ERR_BADVERSION)
        else TRY_CONST(name, CMAE_ERR_BADARG)
        else TRY_CONST(name, CMAE_ERR_MALLOC)
        else TRY_CONST(name, CMAE_ERR_CART_LOAD)
        else TRY_CONST(name, CMAE_ERR_CART_INIT)
        else TRY_CONST(name, CMAE_ERR_INIT)
        else TRY_CONST(name, CMAE_ERR_START)
        else TRY_CONST(name, CMAE_ERR_NOINIT)
        else TRY_CONST(name, CMAE_ERR_NOSTART)
        else TRY_CONST(name, CMAE_ERR_CARTNOSUPP)
        else TRY_CONST(name, CMAE_ERR_SHUTDOWN)
        else TRY_CONST(name, CMAE_ERR_CFG_WARNING)
        else TRY_CONST(name, CMAE_ERR_CFG_ERROR)
        else TRY_CONST(name, CMAE_ERR_CFG_KEY)
        else TRY_CONST(name, CMAE_ERR_CFG_SYNTAX)
        else TRY_CONST(name, CMAE_ERR_CFG_VALUE)
        else TRY_CONST(name, CMAE_ERR_CFG_NULL)
        else TRY_CONST(name, CMAE_ERR_CFG_IMMUTABLE)
        else TRY_CONST(name, CMAE_ERR_CFG_DEPRECATED)
        else TRY_CONST(name, CMAE_ERR_UNKNOWNHOST)
        else TRY_CONST(name, CMAE_ERR_PROTOCOL)
        else TRY_CONST(name, CMAE_ERR_MISCSOCKET)
        else TRY_CONST(name, CMAE_ERR_NOTCONN)
        else TRY_CONST(name, CMAE_ERR_CONNREFUSED)
        else TRY_CONST(name, CMAE_ERR_NETUNREACH)
        else TRY_CONST(name, CMAE_ERR_TIMEDOUT)
        else TRY_CONST(name, CMAE_ERR_AGAIN)
        else TRY_CONST(name, CMAE_ERR_CONNRESET)
        else TRY_CONST(name, CMAE_ERR_UNUSED)
        else TRY_CONST(name, CMAE_ERR_OTHER)
        else die("BUG: invalid Cloudmark::CMAE::Client constant: \"%s\"\n", name);
    OUTPUT:
        RETVAL
