/* $Id: cmae_client.h,v 1.20.4.3 2007/05/29 19:39:18 chip Exp $ */

#ifndef _CLOUDMARK_AUTHORITY_ENGINE_CLIENT_INTERFACE_H_
#define _CLOUDMARK_AUTHORITY_ENGINE_CLIENT_INTERFACE_H_

#include <stddef.h>
#include "cmae_error.h"
#include "cmae_envelope.h"

#if __cplusplus
extern "C" {
#endif


extern const char * const CMAE_Client_Version;


/*
 * Magic cookie describing a CMAE client session.
 *
 * NOTE ON THREADING: All of the functions in this API may be called
 * in a multi-threaded environment, just so long as no more than one
 * thread is using any given client session ("CMAE_Client_Session") at
 * any given time.
 */

struct cmae_client_session;
typedef struct cmae_client_session *CMAE_Client_Session;


/*
 * The default port for client connections, in _host_ order.
 */

CMAE_EXPORT unsigned short CMAE_Client_Default_Port(void);


/*
 * Open a connection to the CMAE server at the given host and port.
 *
 * If ServerName is non-null, it must point to either a hostname
 *   or a string containing an IP address in dot notation.
 * If ServerName is null, ServerIPAddr must be an IP address in
 *   _network_ order (e.g. 'htonl(INADDR_LOOPBACK)').
 *
 * ServerPort must be the port, in _host_ order; passing zero here
 *   means to use CMAE_Client_Default_Port().
 *
 * ConnectTimeout is specified in seconds; passing zero here means
 *   to use the system default TCP timeout.
 *
 * On success, sets *SessionOut to a CMAE_Client_Session value,
 *     and returns zero (CMAE_ERR_NONE).
 * On failure, returns a non-zero error code.
 *
 * NOTE: Many failures will be the result of failed system calls.
 *       It is recommended to clear 'errno' before calling this function,
 *       and to check 'errno' after a failure.
 */

CMAE_EXPORT CMAE_ERR CMAE_Client_Open(CMAE_Client_Session *SessionOut,
                                      const char *ServerName,
                                      unsigned long ServerIPAddr,
                                      unsigned short ServerPort,
                                      unsigned int ConnectTimeout);


/*
 * Close an open CMAE client connection.
 *
 * On success, returns zero.
 * On failure, returns a non-zero error code.
 *
 * NOTE: Many failures will be the result of failed system calls.
 *       It is recommended to clear 'errno' before calling this function,
 *       and to check 'errno' after a failure.
 */

CMAE_EXPORT CMAE_ERR CMAE_Client_Close(CMAE_Client_Session Session);


/*
 * Set and get miscellanous control parameters.
 * On success, returns zero.
 * On failure, returns a non-zero error code.
 *
 * Parameters:
 *    ProtocolTimeout    Timeout for protocol send/receive, in seconds.
 *                       Zero, the default, means no timeout.
 *    MaxMessageSize     Maximum size of message that the server is willing
 *                       to accept; zero means no maximum.
 */

CMAE_EXPORT CMAE_ERR CMAE_Client_SetProtocolTimeout(CMAE_Client_Session Session,
                                                    unsigned int ProtocolTimeout);
CMAE_EXPORT CMAE_ERR CMAE_Client_GetProtocolTimeout(CMAE_Client_Session Session,
                                                    unsigned int *ProtocolTimeoutOut);

CMAE_EXPORT CMAE_ERR CMAE_Client_GetMaxMessageSize(CMAE_Client_Session Session,
                                                   size_t *MaxMessageSizeOut);

/*
 * One-shot scoring function.  Asks the CMAE server whether the given
 * RFC822 message, with the optionally given envelope information, is
 * spam.
 *
 * If you do not wish to score with an envelope, pass NULL for Envelope.
 *
 * If scored successfully the following will be set:
 *   - *ScoreOut to the spam score between 0 (ham) and 100 (spam).
 *   - *CategoryOut and *SubCategoryOut to An approximate category and
 *     sub-category for the message.
 *   - *RescanOut to a flag; if set, it hints that the score may become more
 *     accurate later.
 *   - *AnalysisOut to an allocated text block that should be added to the
 *     message as a header for diagnostic purposes. This should be freed via
 *     a call to CMAE_Client_Free().
 * Any or all of the ...Out pointers may be NULL, which means that the caller is
 * not interested in getting back the given value(s).
 *
 * On failure, return a non-zero error code.
 */

CMAE_EXPORT CMAE_ERR CMAE_Client_Score(CMAE_Client_Session Session,
                                       CMAE_Envelope Envelope,
                                       const char *RFC822Content, size_t RFC822ContentLength,
                                       unsigned int *ScoreOut,
                                       unsigned int *CategoryOut, unsigned int *SubCategoryOut,
                                       unsigned int *RescanOut, char **AnalysisOut);

/*
 * One-shot scoring function.  Takes the block of text returned in
 * AnalysisOut by a call to CMAE_Client_Score.  Asks the CMAE server
 * to score the metainformation describing the message without
 * processing the content again.
 *
 * If scored successfully, the following will be set:
 *   - *ScoreOut to the spam score between 0 (ham) and 100 (spam).
 *   - *CategoryOut and *SubCategoryOut to an approximate category and
 *     sub-category for the message, or zero if not known.
 *   - *RescanOut to a flag; if set, it hints that the score may become more
 *     accurate later.
 * Any or all of the ...Out pointers may be NULL, which means that the caller is
 * not interested in getting back the given value(s).
 *
 * On failure, return a non-zero error code.
 */
CMAE_EXPORT CMAE_ERR CMAE_Client_Score_Analysis(CMAE_Client_Session Session,
                                                const char *AnalysisText,
                                                unsigned int *ScoreOut, unsigned int *CategoryOut,
                                                unsigned int *SubCategoryOut, unsigned int *RescanOut);

/*
 * Return a text description of a category and/or sub-category as an
 * allocated string.
 *
 * Either of CategoryDescOut/SubCategoryDescOut can be NULL, meaning
 * that no description will be returned for the category or
 * sub-category.
 *
 * On failure, return a non-zero error code.
 */

CMAE_EXPORT CMAE_ERR CMAE_Client_DescribeCategory(CMAE_Client_Session Session,
                                                  unsigned int Category, unsigned int SubCategory,
                                                  char **CategoryDescOut, char **SubCategoryDescOut);

/*
 * Get the version of the CMAE cartridge running in the server.
 *
 * On success, sets *VersionOut to an allocated string containing the
 * version, and returns zero.  User must call CMAE_Client_Free() on
 * the value of *VersionOut to avoid a memory leak.
 *
 * On failure, returns a non-zero error code.
 */

CMAE_EXPORT CMAE_ERR CMAE_Client_GetCartridgeVersion(CMAE_Client_Session Session,
                                                     char **VersionOut);

/*
 * Free memory allocated by CMAE Client library.
 */

CMAE_EXPORT void CMAE_Client_Free(void *allocated_memory);

#if __cplusplus
} /* extern "C" */
#endif

#endif /* _CLOUDMARK_AUTHORITY_ENGINE_CLIENT_INTERFACE_H_ */
