#
# This is the Makefile generator for the Cloudmark::CMAE::Client module.
#
# It should be run with normal MakeMaker command line arguments, as well
# as this special one:
#
#    CMAE=<path_to_CMAE_SDK>
#
# The given path should include the CMAE SDK headers (cmae_client.h etc.)
# as well as the CMAE client library (libcmaeclient.so).
#

use 5.006_001;
use strict;
use ExtUtils::MakeMaker;
use File::Spec;
use Cwd;

my $CMAE;
for my $i (0 .. $#ARGV) {
    if ($ARGV[$i] =~ /^CMAE=(.*)/s) {
        $CMAE = $1;
        splice @ARGV, $i, 1;
        last;
    }
}

#
# Locate CMAE headers and library with "CMAE=" parameter or in local
# tree (cloudmark CVS).
#

chomp(my $cm_engine = `cd ../.. && pwd`);

my (@cm_include_dirs, $cm_lib_dir);

if (defined $CMAE) {
    # build using a Cloudmark distribution

    @cm_include_dirs = ("$CMAE/include");
    $cm_lib_dir      = "$CMAE/lib";

    die "There is no CMAE SDK in directory '$CMAE'\n"
      unless -e "$cm_include_dirs[0]/cmae_client.h" and -e "$cm_lib_dir/libcmaeclient.so";
}
elsif (-e "$cm_engine/bolo/cmae_client.h" and -e "$cm_engine/bolo/libcmaeclient.so") {
    # build inside Cloudmark source tree

    @cm_include_dirs = ( "$cm_engine/bolo", "$cm_engine" );
    $cm_lib_dir      = "$cm_engine/bolo";

    # check various copies of the CMAE versio here, and die now if
    # they don't all match, so that a 'make' will fail
    eval { check_versions() };
    if ($@)
      { die "Error: File versions are out of sync: $@" }
}
else {
    die "Please rerun with argument of CMAE=<path to CMAE SDK>\n";
}


# See lib/ExtUtils/MakeMaker.pm for details of how to influence
# the contents of the Makefile that is written.

WriteMakefile(
    NAME              => 'Cloudmark::CMAE::Client',
    VERSION_FROM      => 'lib/Cloudmark/CMAE/Client.pm', # finds $VERSION

    ($] >= 5.008
     ? (NO_META       => 1) # don't create META.yml
     : ()),

    PREREQ_PM         => {
                          # e.g., Module::Name => 1.1
                          'Params::Validate' => 0, # any
                         },

    ($] >= 5.005
     ? (ABSTRACT_FROM => 'lib/Cloudmark/CMAE/Client.pm', # retrieve abstract from module
        AUTHOR        => 'Chip Salzenberg <chip@cloudmark.com>')
     : ()),

    DEFINE            => '',

    INC               => join(' ', map "-I$_", @cm_include_dirs),

    LIBS              => ["-L$cm_lib_dir -lcmaeclient"],
    'dist'            => { TAR      => ($^O eq 'freebsd' || $^O eq 'solaris') ? 'gtar' : 'tar',
                           TARFLAGS => '--owner=0 --group=0 -cvf',
                           COMPRESS => "gzip -f --best" },

    # Un-comment this if you add C files to link with later:
    # OBJECT            => '$(O_FILES)', # link all the C files too
);

# MakeMaker likes to use rpath (LD_RUN_PATH).  We don't want it to do
# that, since the current and final locations of libcmaeclient may differ.
# So, instead, we arrange to set LD_LIBRARY_PATH in Perl's environment
# during the build and install process.
{
    package MY;

    sub const_loadlibs {
        my $self = shift;
        my $rpath = delete $self->{LD_RUN_PATH};
        my $s = $self->SUPER::const_loadlibs;
        if ($rpath) {
            $s .= <<EOF;
PERLRUN_ENV = env LD_LIBRARY_PATH=\$(LD_LIBRARY_PATH):$rpath
EOF
        }
        $s;
    }

    sub init_PERL {
        my $self = shift;
        $self->SUPER::init_PERL;
        foreach my $perl (qw(PERL FULLPERL ABSPERL)) {
            my $run  = $perl.'RUN';
            $self->{$run} = "\$(PERLRUN_ENV) $self->{$run}";
        }
        return 1;
    }
}


# On our solaris machines, gcc was configured to use solaris ld, and
# as a result doesn't understand the -Wl,-E option.
if ($^O eq "solaris") {
    local $^I = '';
    @ARGV = qw( Makefile );
    while (<>) {
        s/-Wl,-E//g;
        print;
    }
}

#----------------------------------------------------------------
# When building in CMAE source tree, verify version consistency:
#   cmae_version.h
#   $Cloudmark::CMAE::Client::VERSION
#   README

sub check_versions {

    # 1. find CMAE version

    my $engine_dir = do {
        my $pwd = getcwd;
        $pwd =~ m{(.*/cloudmark)/}
          or die "Current directory doesn't include '/cloudmark/'\n";
        "$1/spilter/engine";
    };

    my ($CMAE_VERSION, @CMAE_VERSION) = do {
        my $version_h = "$engine_dir/cmae_version.h";
        open my $fh, '<', $version_h
          or die "Can't open $version_h: $!\n";
        local $_ = do { local $/; <$fh> };

        my @hv = m{ ^ \s* \# \s* define \s+ CMAE_VERSION \s+
                    " ( (\d+) \. (\d+) \. (\d+) \. (\d+) ) (?:-eval)? " }xm
          or die "Can't parse CMAE_VERSION in $version_h\n";
        @hv;
    };

    # 2. check Cloudmark::CMAE::Client version

    {
        my $pm = 'lib/Cloudmark/CMAE/Client.pm';
        open my $fh, '<', $pm
          or die "Can't open $pm: $!\n";
        my $pm_vers;
        while (<$fh>) {
            last if ($pm_vers) = m{ ^ \s* \$VERSION \s* = \s* [\'\"](.*?)[\'\"] }x;
        }
        $pm_vers
          or die "Can't parse \$VERSION in $pm\n";

        # strip the build number.  and an extra digit is OK - if it's
        # there, it's an intermediate version of the Perl module
        $pm_vers =~ s{ ( \.\d{5} ) \d{2,3} $ }{$1}x;

        my $hv = sprintf "%d.%03d%02d", @CMAE_VERSION[0..2];
        $pm_vers eq $hv
          or die "$pm \$VERSION ($pm_vers) does not match cmae_version.h ($hv)\n";
    }

    # 3. check README version

    {
        my $readme = `sed 1q README`;
        $readme =~ m{ \b \Q$CMAE_VERSION\E \s* $ }x
          or die "README version does not match cmae_version.h ($CMAE_VERSION)\n";
    }
}

