package Cloudmark::CMAE::Client;

our $VERSION;
BEGIN {
    $VERSION = '2.0000329';  # 2.0.3.29
}

use 5.006_001;
use strict;
use warnings;
use Carp;
use Socket qw( inet_aton );

#
# Set up parameter validation.
#

use Params::Validate qw( :all );
use Scalar::Util qw( looks_like_number );

# Locked hashes are helpful, but are not available (per se) in Perl
# 5.6.  Some validations are not available there.  But that's OK, the
# coding errors they are designed to detect would be found by tests
# run under 5.8.

BEGIN {
    if ($] >= 5.008) {
        require Hash::Util;
        Hash::Util->import(qw( lock_keys ));
    }
    else {
        *lock_keys = sub (\%@) {};
    }
}

#
# Helper data for validate() calls
#

my $rx_integer = qr{ ^ \s* \d+ \s* $ }x;
my @cb_number = ( number => sub { looks_like_number(shift) } );
my @cb_ipaddr = ( ipaddr => sub { $_[0] =~ qr{ ^ \d+ \. \d+ \. \d+ \. \d+ $ }x
                                   && inet_aton($_[0]) } );

#
# Load XS portion during compilation.
# We do this early so we can know the valid constants early,
# in turn so that we can know our valid exports early.
#

BEGIN {
    require XSLoader;
    XSLoader::load('Cloudmark::CMAE::Client', $VERSION);
}

#
# Set up exporting during compilation.  This is a good general
# practice to avoid difficulties with mutual dependency.
#

my @Constants;
our (@EXPORT, @EXPORT_OK, %EXPORT_TAGS);

BEGIN {
    require Exporter;
    *import = \&Exporter::import;

    my @errconst =
      grep { defined eval { _constant($_) } }
        qw(
          CMAE_ERR_NONE
          CMAE_ERR_BADVERSION
          CMAE_ERR_BADARG
          CMAE_ERR_MALLOC
          CMAE_ERR_CART_LOAD
          CMAE_ERR_CART_INIT
          CMAE_ERR_INIT
          CMAE_ERR_START
          CMAE_ERR_NOINIT
          CMAE_ERR_NOSTART
          CMAE_ERR_CARTNOSUPP
          CMAE_ERR_SHUTDOWN
          CMAE_ERR_CFG_WARNING
          CMAE_ERR_CFG_ERROR
          CMAE_ERR_CFG_KEY
          CMAE_ERR_CFG_SYNTAX
          CMAE_ERR_CFG_VALUE
          CMAE_ERR_CFG_NULL
          CMAE_ERR_CFG_IMMUTABLE
          CMAE_ERR_CFG_DEPRECATED
          CMAE_ERR_UNKNOWNHOST
          CMAE_ERR_PROTOCOL
          CMAE_ERR_MISCSOCKET
          CMAE_ERR_NOTCONN
          CMAE_ERR_CONNREFUSED
          CMAE_ERR_NETUNREACH
          CMAE_ERR_TIMEDOUT
          CMAE_ERR_AGAIN
          CMAE_ERR_CONNRESET
          CMAE_ERR_OTHER
          CMAE_ERR_UNUSED
        );

    @EXPORT_OK = ( @errconst, qw( CMAE_StrError ) );
    %EXPORT_TAGS = (
                    errors => [ @errconst, qw( CMAE_StrError ) ],
                    all    => [ @EXPORT, @EXPORT_OK ],
                   );

    push @Constants, @errconst;

    #
    # Define constant subs.
    # 5.6 doesn't have C<use constant HASHREF>,
    #  so we have to do this the hard way
    #

    eval join '', map { my $v = _constant($_); "sub $_ () { $v }\n" } @Constants;
    die "Can't compile constants: $@" if $@;
}


#----------------------------------------------------------------
# Class data and methods
#

sub library_version {
    my $class = shift;
    validate @_, {};

    _Library_Version();
}

sub default_port {
    my $class = shift;
    validate @_, {};

    _Default_Port();
}

our $New_Error;

sub last_new_error {
    my $class = shift;
    validate @_, {};

    $New_Error;
}


#----------------------------------------------------------------
# Ctor/Dtor
#

sub new {
    my $class = shift;
    my %p = validate @_, { host    => 1,
                           port    => { default => 0, regex => $rx_integer },
                           timeout => { default => 0, callbacks => { @cb_number } } };
    lock_keys %p;

    my $session;
    my $err = _Open($session, $p{host}, 0, $p{port}, $p{timeout});

    $New_Error = $err;

    wantarray
      ? ($session, $err)
      : $session;
}

sub DESTROY {
    _Close($_[0]);
}


#----------------------------------------------------------------
# Object methods
#

#
# manipulate protocol timeout
#

sub get_protocol_timeout {
    my $self = shift;
    my %p = validate @_, { out_timeout => { type => SCALARREF } };
    lock_keys %p;

    $self->_GetProtocolTimeout(${ $p{out_timeout} });
}

sub set_protocol_timeout {
    my $self = shift;
    my %p = validate @_, { timeout => { callbacks => { @cb_number } } };
    lock_keys %p;

    $self->_SetProtocolTimeout($p{timeout});
}

#
# get max size of message that can be sent
#

sub get_max_message_size {
    my $self = shift;
    my %p = validate @_, { out_size => { type => SCALARREF } };
    lock_keys %p;

    $self->_GetMaxMessageSize(${ $p{out_size} });
}

#
# score a message
#

sub score {
    my $self = shift;

    #
    # OK, this is incredibly hacky, but it's necessary to avoid making
    # another copy of the (potentially quite large) message string.
    #
    # We find the value while it's still sitting in @_, take a reference
    # to it, then remove it (and its key) from @_.
    #
    # If we found it, then we don't tell validate() to look for it.
    # Conversely, if we *didn't* find it, we *do* tell validate() to look
    # for it, knowing it will fail and trigger the appropriate error.
    #
    my $rfc822_ref;
    for (my $i = 0; $i < @_; ) {
        if ($_[$i] eq 'rfc822') {
            $rfc822_ref = \$_[$i + 1];
            splice @_, $i, 2;
        }
        else {
            $i += 2;
        }
    }

    my $never_returned;

    my %p = validate @_, { $rfc822_ref ? () : (rfc822 => 1),
                           envelope  => { optional => 1, type => HASHREF },
                           out_score => { type => SCALARREF },
                           out_category => { default => \$never_returned, type => SCALARREF },
                           out_sub_category => { default => \$never_returned, type => SCALARREF },
                           out_rescan => { default => \$never_returned, type => SCALARREF },
                           out_analysis => { default => \$never_returned, type => SCALARREF } };

    lock_keys %p, qw( envelope out_score out_category out_sub_category out_rescan out_analysis );

    my %env;
    if ($p{envelope}) {
        %env = validate_with(
                  params => [ %{ $p{envelope} } ],
                  spec   => { FromIP   => { optional => 1, callbacks => { @cb_ipaddr } },
                              FromHost => { optional => 1 },
                              Helo     => { optional => 1 },
                              MailFrom => { optional => 1 },
                              RcptTo   => { optional => 1, type => ARRAYREF } },
                  called => "CMAE envelope attributes"
                );
    }

    $self->_Score(\%env, $$rfc822_ref, ${ $p{out_score} },
                  ${ $p{out_category} }, ${ $p{out_sub_category} },
                  ${ $p{out_rescan} }, ${ $p{out_analysis} });
}

#
# re-score a message based on its analysis header
#

sub score_analysis {
    my $self = shift;

    my $never_returned;

    my %p = validate @_, { analysis         => 1,
                           out_score        => { type => SCALARREF },
                           out_category     => { default => \$never_returned, type => SCALARREF },
                           out_sub_category => { default => \$never_returned, type => SCALARREF },
                           out_rescan       => { default => \$never_returned, type => SCALARREF } };

    lock_keys %p, qw( analysis out_score out_category out_sub_category out_rescan );

    $self->_Score_Analysis($p{analysis},
                           ${ $p{out_score} },
                           ${ $p{out_category} }, ${ $p{out_sub_category} },
                           ${ $p{out_rescan} });
}

#
# Return the description for a category and/or sub-category
#

sub describe_category {
    my $self = shift;

    my $never_returned;

    my %p = validate @_, { category => 1,
                           sub_category => 1,
                           out_category_desc => { default => \$never_returned, type => SCALARREF },
                           out_sub_category_desc => { default => \$never_returned, type => SCALARREF } };

    lock_keys %p, qw( category sub_category out_category_desc out_sub_category_desc );

    $self->_DescribeCategory($p{category}, $p{sub_category},
                             ${ $p{out_category_desc} }, ${ $p{out_sub_category_desc} });

}

#
# Get the version of the cartridge that the server is using.
#

sub get_cartridge_version {
    my $self = shift;

    my %p = validate @_, { out_version => { type => SCALARREF } };

    $self->_GetCartridgeVersion(${ $p{out_version} });
}

1;
__END__

=head1 NAME

Cloudmark::CMAE::Client - Perl interface to CMAE client library

=head1 SYNOPSIS

  use Cloudmark::CMAE::Client qw( :errors );

  {

    # 1. Establish connection

    my ($client, $err) = Cloudmark::CMAE::Client->new(
                                                       host    => 'bob',
                                                       port    => 13013,  # OPTIONAL
                                                       timeout => 60,     # OPTIONAL
                                                     );
    if ($err) {
      # handle error, or else:
      die "Can't connect to CMAE server at bob:13013: " . CMAE_StrError($err) . "\n";
    }


    # 2. Optionally manipulate protocol timeout

    my $old_timeout;
    $err = $client->get_protocol_timeout(out_timeout => \$old_timeout);
    print "old timeout was $old_timeout, setting timeout to 60\n";
    $client->set_protocol_timeout(timeout => 60);


    # 3a. Score message without envelope information.

    my $rfc822 = ...;   # get message text

    my $score;
    my $err = $client->score(rfc822 => $rfc822, out_score => \$score,
                             out_category => \$category, out_sub_category => \$sub_category,
                             out_rescan => \$rescan, out_analysis => \$analysis);
    if ($err) {
        # handle error, or else:
        if ($err == CMAE_ERR_PROTOCOL)
          { die "Looks like the server closed our connection\n" }
        else
          { die "Can't get score: " . CMAE_StrError($err) . "\n" }
    }


    # 3b. Score message with envelope information
    #     (All envelope entries are optional; an empty hash is thus legal.)

    my %env = (
                FromIP   => '131.107.1.6',
                FromHost => $orig_host,
                Helo     => $helo_string,
                MailFrom => $from_addr,
                RcptTo   => [ @to_addrs ],
              );

    my ($category, $sub_category, $rescan, $analysis);

    my $err = $client->score(rfc822 => $rfc822, envelope => \%env, out_score => \$score,
                             out_category => \$category, out_sub_category => \$sub_category,
                             out_rescan => \$rescan, out_analysis => \$analysis);


    # 3c. Map category numbers to descriptions
    #
    my ($cat_desc, $sub_cat_desc);

    my $err = $client->describe_category(category => $category, sub_category => $sub_category,
                                         out_category_desc => \$cat_desc,
                                         out_sub_category_desc => \$sub_cat_desc);



    # 4. Allow client object to go out of scope or otherwise delete
    #    all references to it; it will close automatically
  }


=head1 DESCRIPTION

The CMAE server is a daemon that provides a network service to score the
spamminess of email messages.  This module, Cloudmar::CMAE::Client,
provides a Perl interface to the CMAE client library, allowing a Perl
program to use that service.

The Cloudmark::CMAE::Client module provides an object interface that
represents a connection to a CMAE server; class methods that provide
module information; and exportable utility subroutines for error handling.

=head1 EXPORTS

The Cloudmark::CMAE::Client module can export all valid CMAE_ERR_* error
contants.  See the CMAE SDK documentation for the current list of error
codes and meanings.

The module can also export the function &CMAE_StrError, which transforms a
valid CMAE error code to its corresponding error messages.

These exportable subroutines may be named specifically in the C<use>
statement of this module.  Alternatively the Exporter tag ":errors" will
select all errors codes plus &CMAE_StrError.  The Exporter tag ":all" is
provided for future expansion; it is currently synonymous with ":error".

=head1 CLASS METHODS

=over 4

=item B<library_version()>

Returns the version of the CMAE client library ("libcmaeclient.so") that
the current program is using.

=item B<default_port()>

Returns the default CMAE server port number that will be used if no port
number is given, or if the given port number is zero.

=back

=head1 OBJECT METHODS

=over 4

=item new(host => $hostname, port => $portnum, timeout => $timeout)

Create a new connection to the CMAE server at host $host.  The port number
is optional; if not given, or given as zero, it defaults to the value
returned by the default_port() class method (see above).  The timeout is
also optional; if not given, or given as zero, it defaults to the standard
socket connection timeout.

In a I<scalar> context, the return value is a new Cloudmark::CMAE::Client
object on success, or undef on failure.  The error code associated with a
failure can be obtained with the last_new_error() class method (see
above).

In a I<list> context, the return values are [0] the scalar return value as
described above, and [1] the CMAE error code, or zero (CMAE_ERR_NONE) on
success.

=item B<DESTROY()>  I<[do not call this]>

Close connection as part of object destruction.  DO NOT CALL THIS FUNCTION
DIRECTLY.  Perl will take care of it.

=item B<get_protocol_timeout(out_timeout => \$old_timeout)>

Get the current protocol timeout (i.e. how long CMAE Client will wait for
a response after sending a request) into the given variable.

Return CMAE error code, or zero (CMAE_ERR_NONE) on success.

=item B<set_protocol_timeout(timeout => $new_timeout)>

Set the protocol timeout (i.e. how long CMAE Client will wait for a
response after sending a request) to the given value.

Return CMAE error code, or zero (CMAE_ERR_NONE) on success.

=item B<get_max_message_size(out_size => \$size)>

Get the maximum message size that the server is willing to accept for
scoring.  A value of zero means no maximum (within system resource
limits).

Messages larger than the maximum size size should be truncated before
scoring, or else not scored.  Attempts to score messages larger than
this size may fail and/or cause disconnection from the server.

Return CMAE error code, or zero (CMAE_ERR_NONE) on success.

=item B<score(rfc822 => $rfc822_msg, envelope => { .. }, out_score => \$score, out_category => \$category, out_sub_category => \$sub_category, out_rescan => \$rescan, out_analysis => \$analysis)>

Ask the server for the spam score (0-100) of the given RFC822 message
with the given (optional) envelope inforomation.  Store the score into
the given variable. All other return values are optional. If provided
they return additional information about the message.

The envelope hash is optional.  Each of the elements of that hash is also
optional.  The legal envelope attributes are:

    FromIP      Source IP address as a dotted quad string, e.g. '131.107.1.6'
    FromHost    Source host name (e.g. from reverse DNS lookup)
    Helo        HELO string
    MailFrom    Source mail address, from SMTP "MAIL FROM"
    RcptTo      Array of destination mail addresses, from SMTP "RCPT TO"

Return CMAE error code, or zero (CMAE_ERR_NONE) on success.

=item B<score_analysis(analysis => $analysis, out_score => \$score, out_category => \$category, out_sub_category => \$sub_category, out_rescan => \$rescan)>

Ask the server for the spam score (0-100) of the given analysis
header.  Store the score into the given variable.  All other return
values are optional.  If provided they return additional information
about the message.

Return CMAE error code, or zero (CMAE_ERR_NONE) on success.

=item B<describe_category(category => $category_number, sub_category => $sub_category_number, out_category_desc => \$category_desc, out_sub_category_desc => \$sub_category_desc);

Ask the server for text strings that describe the provided category
and/or sub-category. The category and sub-category are both required.
If you only want a description for a category, provide 0 for the
sub-category. If provided, scalar references will be filled in with
descriptions of the category and sub-category.

Return CMAE error code, or zero (CMAE_ERR_NONE) on success.

=item B<get_cartridge_version(out_version => $version)>

Ask the server for the version of the SpamDNA cartridge it is using,
and assign that version string to C<$version>.

Return CMAE error code, or zero (CMAE_ERR_NONE) on success.

=back

=head1 ERROR HANDLING

If any CloudmarK::CMAE::Client subroutine or method is called with
incorrect or missing parameters, it will throw an exception ("die").
If your code builds parameter lists dynamically, and runs in a
situation where death is not a viable option (e.g. a long-running
daemon), you may wish to wrap such calls in C<eval BLOCK> to catch and
handle any exceptions.

=head1 DEPENDENCIES

This module depends on the CMAE SDK, which contains the CMAE client
library and associated C header files.  The header files are required
at module build time; the library alone is required at runtime.

This module also depends on the following CPAN modules:

   Params::Validate

=head1 SEE ALSO

CMAE Client/Server documentation.

=head1 AUTHOR

Chip Salzenberg, E<lt>chip@cloudmark.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright 2006 by Cloudmark, Inc.

=cut
